#include "speex/speex_bits.h"

static int speex_skip_wb_frame(SpeexBits *bits)
{
   static int SpeexWBSubModeSz[] = {0, 36, 112, 192, 352, 0, 0, 0 };
   unsigned int c;

   /* skip up to two wideband frames */
   if (  (speex_bits_remaining(bits) >= 5)
       && speex_bits_unpack_unsigned(bits, 1))
   {
      c = speex_bits_unpack_unsigned(bits, 3);
      speex_bits_advance(bits, SpeexWBSubModeSz[c]);

      if ( (speex_bits_remaining(bits) >= 5)
         && speex_bits_unpack_unsigned(bits, 1))
      {
         c = speex_bits_unpack_unsigned(bits, 3);
         speex_bits_advance(bits, SpeexWBSubModeSz[c]);

         if ( (speex_bits_remaining(bits) >= 5)
            && speex_bits_unpack_unsigned(bits, 1))
         {
            /* too many in a row */
            printf("\tCORRUPT too many wideband streams in a row\n");
            return -1;
         }
      }

   }
   return 0;
}

static int speex_get_num_samples_nb(SpeexBits *bits)
{
   static int SpeexSubModeSz[] = {0, 43, 119, 160, 220, 300, 364, 492,
                                  79, 0, 0, 0, 0, 0, 0, 0 };
   static int SpeexInBandSz[] = {1, 1, 4, 4, 4, 4, 4, 4,
                                 8, 8, 16, 16, 32, 32, 64, 64 };
   int sample_count = 0;
   unsigned int c;

   printf("speex_get_samples(%d)\n", speex_bits_nbytes(bits));
   while (speex_bits_remaining(bits) >= 5) {
      /* skip wideband frames */
      if (speex_skip_wb_frame(bits) < 0)
      {
         printf("\tERROR reading wideband frames\n");
         break;
      }

      if (speex_bits_remaining(bits) < 4)
      {
         printf("\tERROR not enough bits left after wb\n");
         break;
      }

      /* get control bits */
      c = speex_bits_unpack_unsigned(bits, 4);
      printf("\tCONTROL: %d at %d:%d\n", c, bits->charPtr, bits->bitPtr);

      if (c == 15){
         printf("\tTERMINATOR\n");
         break;
      } else if (c == 14) {
         /* in-band signal; next 4 bits contain signal id */
         c = speex_bits_unpack_unsigned(bits, 4);
         printf("\tIN-BAND %d bits\n", SpeexInBandSz[c]);
         speex_bits_advance(bits, SpeexInBandSz[c]);
      } else if (c == 13) {
         /* user in-band; next 5 bits contain msg len */
         c = speex_bits_unpack_unsigned(bits, 5);
         printf("\tUSER-BAND %d bytes\n", c);
         speex_bits_advance(bits, c * 8);
      } else if (c > 8) {
         printf("\tUNKNOWN sub-mode %d\n", c);
         break;
      } else {
         /* NB frame; skip number bits for submode (less the 5 control bits) */
         printf("\tSUBMODE %d %d bits\n", c, SpeexSubModeSz[c]);
         speex_bits_advance(bits, SpeexSubModeSz[c] - 5);

         sample_count += 160;
      }
   }
   printf("\tSAMPLES: %d\n", sample_count);
   return sample_count;
}